@@ -4,7 +4,7 @@
  *              to build a moderately scalable and highly available server
  *              based on a cluster of servers, with more flexibility.
  *
- * Version:     $Id: tcp_vs.c,v 1.15.2.2 2004/12/08 13:58:13 wensong Exp $
+ * Version:     $Id: tcp_vs.c,v 1.15.2.3 2004/12/17 17:39:48 wensong Exp $
  *
  * Authors:     Wensong Zhang <wensong@linuxvirtualserver.org>
  *
@@ -189,15 +189,20 @@ tcp_vs_relay_socket(struct socket *from, struct socket *to)
 	struct sock *sk;
 	struct tcp_opt *tp;
 
+	lock_sock(from->sk);
 	skb = skb_dequeue(&from->sk->receive_queue);
-	if (!skb)
+	if (!skb) {
+		release_sock(from->sk);
 		return -1;
+	}
 
 	len = skb->len;
 	if (len == 0) {
 		kfree_skb(skb);
+		release_sock(from->sk);
 		return 0;
 	}
+	release_sock(from->sk);
 
 	if (!sysctl_ktcpvs_zerocopy_send) {
 		int res;
